#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb 13 15:55:39 2021

@author: anton
"""

from SLiCAP import *

files = ['ABconcept', 'CDconcept']
# In all circuit files:
# signal source: V1, value = 0 if gainType='gain' or value = 1 if gainType='vi'
# R1 model = 'r' value = 'R_ell'
# The name of the input node of the amplifier is 'i'
# The name of the output node of the amplifier is 'o'

prj = initProject('Active antenna T1 matrix solutions')

# Target values for the output impedance z_o, the source-to-load voltage 
# transfer 'A_v', the load resistance 'R_L' and the antenna capacitance 'CA':

z_o = 50
A_v = 1
R_L = 50
CA  = 6.3e-12
Rs  = 1

# Definition of symbolic variables that will be used in expressions:
z_s = sp.sympify('1/s/C_A')
A, B, C, D, R_ell, C_A, R_s = sp.symbols('A, B, C, D, R_ell, C_A, R_s')

# The program
for fileName in files:
    #makeNetlist(fileName + '.asc', fileName)
    i1 = instruction()
    i1.setCircuit(fileName + '.cir')
    i1.setSimType('symbolic')
    i1.setGainType('gain')
    i1.setDataType('laplace')
    
    i1.setSource('V1')
    
    # determination of Vi
    # Vi is the transfer from V1 to the voltage V_i at node i
    i1.setDetector('V_i')
    result = i1.execute()
    V_i = result.laplace
    
    # determination of Ii
    # Ii is the transfer from V1 to the current through V1
    i1.setDetector('I_V1')
    result = i1.execute()
    I_i = -result.laplace
    
    # determination of Vo
    # Vo is the transfer from V1 to the voltage V_o at node o
    i1.setDetector('V_o')
    result = i1.execute()
    V_o = result.laplace
    
    # determination of Io
    # Io is the transfer from V1 to the current through R1
    i1.setDetector('I_R1')
    result = i1.execute()
    I_o = result.laplace
    
    htmlPage('Determination of T1 matrix parameters')
    head2html('Test circuit')
    img2html(fileName + '.svg', 600)
    
    
    # Calculate the T1 parameters
    T_A = sp.simplify(sp.limit(V_i/V_o, R_ell, 'oo'))
    T_B = sp.simplify(sp.limit(V_i/I_o, R_ell, 0))
    T_C = sp.simplify(sp.limit(I_i/V_o, R_ell, 'oo'))
    T_D = sp.simplify(sp.limit(I_i/I_o, R_ell, 0))
    T1s = sp.Matrix([[T_A, T_B], [T_C, T_D]])
    
    # Print the T1 matrix on the html page
    head2html('T1 matrix of the Device Under Test (DUT)')
    text2html('The T1 matrix of the device under test is found as:')
    eqn2html('T_1', T1s)
    
    # Print the matrix equation on the html page
    text2html('The matrix equation of the DUT is found as:')
    ViIi = sp.Matrix([[sp.Symbol('V_i')], [sp.Symbol('I_i')]])
    VoIo = sp.Matrix([[sp.Symbol('V_o')], [sp.Symbol('I_o')]])
    eqn2html(ViIi, T1s*VoIo)
    
    # Print the source-load transfer on the html page
    text2html('The source-to-load transfer is obtained as:')
    eqn2html('A_v', V_o)
    
    # Print the input impedance of the DUT on the html page
    text2html('The input impedance of the DUT is found as:')
    zi = sp.simplify((T_A*R_ell + T_B)/(T_C*R_ell + T_D))
    eqn2html('z_i', zi)
    
    # Print the output impedance of the DUT on the html page
    text2html('The output impedance of the DUT is found as:')
    zo = sp.simplify((T_D*z_s + T_B)/(T_C*z_s + T_A))
    eqn2html('z_o', zo)
    
    # Determine the numeric values of the matrix coefficients, the source-load
    # transfer, and the output impedance.
    
    # Substitute the known parameters in the expressions for V_o and zo
    V_o = V_o.subs([(R_ell, R_L), (C_A, CA), (R_s, Rs)])
    zo  = zo.subs([(R_ell, R_L), (C_A, CA), (R_s, Rs)])
    # Solve the unknown variables; this will return a dictionary with key-value
    # pairs. The keys are the  names of the unknown variables
    sols = sp.solve([V_o-A_v, zo-z_o])

    if T_A != 0:
        T_A = sols[A]
    if T_B != 0:
        T_B = sols[B]
    if T_C != 0:
        T_C = sols[C]
    if T_D != 0:
        T_D = sols[D]

    # Define the numeric T1 matrix
    T1n = sp.Matrix([[T_A, T_B], [T_C, T_D]])
    
    # Print the numeric values on the html page
    head2html('Numeric values')
    text2html('The numeric values are obtained after solving the equations for $A_v$ and $z_o$ for the target values given below.')
    text2html('The target value of the source-load voltage transfer $A_v$ is:')
    eqn2html('A_v', A_v)
    text2html('The target value of the output impedance $z_o$ is:')
    eqn2html('z_o', z_o)
    text2html('The antenna capacitance $C_A$ equals:')
    eqn2html('C_A', CA)
    text2html('The required parameters of the T1 matrix of the DUT are obtained as:')
    eqn2html('T_1', T1n)
    
    """
    https://stackoverflow.com/questions/23449665/generate-many-sympy-expressions-using-the-same-symbols
    """